@@ -2,7 +2,7 @@ package Geo::Coder::Google;
 
 use strict;
 use warnings;
-our $VERSION = '0.11_01';
+our $VERSION = '0.11_02';
 
 sub new {
     my ($self, %param) = @_;
@@ -28,9 +28,4 @@ See L<Geo::Coder::Google::V2> for V2 API usage.
 
 See L<Geo::Coder::Google::V3> for V3 API usage.
 
-=head2 new
-
-  my $geocoder=Geo::Coder::Google->new(apiver=>2);
-  my $geocoder=Geo::Coder::Google->new(apiver=>3);
-
 =cut
