@@ -2,7 +2,7 @@ package Geo::Coder::Google::V3;
 
 use strict;
 use warnings;
-our $VERSION = '0.11_01';
+our $VERSION = '0.11_02';
 
 use Carp;
 use Encode;
@@ -14,6 +14,7 @@ use URI;
 sub new {
     my($class, %param) = @_;
 
+    my $ua       = delete $param{ua}       || LWP::UserAgent->new(agent => __PACKAGE__ . "/$VERSION");
     my $host     = delete $param{host}     || 'maps.googleapis.com';
 
     my $language = delete $param{language} || delete $param{hl};
@@ -24,27 +25,23 @@ sub new {
     my $key      = delete $param{key}      || '';
    
     bless { 
-        host => $host, language => $language,
+        ua => $ua, host => $host, language => $language, 
         region => $region, oe => $oe, sensor => $sensor,
         client => $client, key => $key,
     }, $class;
 }
 
 sub ua {
-    my $self    = shift;
-    $self->{ua} = shift if @_;
-    $self->{ua} = LWP::UserAgent->new(agent => __PACKAGE__ . "/$VERSION")
-        unless defined $self->{ua};
-    return $self->{ua};
+    my $self = shift;
+    if (@_) {
+        $self->{ua} = shift;
+    }
+    $self->{ua};
 }
 
 sub geocode {
     my $self = shift;
 
-    $self->{"last_res"}    = undef;
-    $self->{"last_data"}   = undef;
-    $self->{"last_status"} = undef;
-
     my %param;
     if (@_ % 2 == 0) {
         %param = @_;
@@ -73,35 +70,36 @@ sub geocode {
         $query_parameters{client} = $self->{client};
         $uri->query_form(%query_parameters);
 
-        my $signature = $self->_make_signature($uri);
+        my $signature = $self->make_signature($uri);
         # signature must be last parameter in query string or you get 403's
         $url = $uri->as_string;
         $url .= '&signature='.$signature if $signature;
     }
 
-    $self->{"last_res"} = $self->ua->get($url);
+    my $res = $self->{ua}->get($url);
 
-    if ($self->{"last_res"}->is_error) {
-        Carp::croak("Google Maps API returned error: " . $self->{"last_res"}->status_line);
+    if ($res->is_error) {
+        Carp::croak("Google Maps API returned error: " . $res->status_line);
     }
 
-    $self->{"last_data"}   = $self->_json->decode($self->{"last_res"}->content);
-    $self->{"last_status"} = $self->{"last_data"}->{"status"};
+    my $json = JSON->new->utf8;
+    my $data = $json->decode($res->content);
 
-    my @results = @{ $self->{"last_data"}->{results} || [] };
+    if ($data->{status} eq 'OVER_QUERY_LIMIT') {
+        Carp::croak("Google Maps API exceeded query limit");
+    } elsif ($data->{status} eq 'REQUEST_DENIED') {
+        Carp::croak("Google Maps API request denied");
+    } elsif ($data->{status} eq 'INVALID_REQUEST') {
+        Carp::croak("Google Maps API invalid request");
+    }
+
+    my @results = @{ $data->{results} || [] };
     wantarray ? @results : $results[0];
 }
 
-sub _json {
-    my $self=shift;
-    $self->{"json"}=JSON->new->utf8
-        unless defined $self->{"json"};
-    return $self->{"json"};
-}
-
 # methods below adapted from 
 # http://gmaps-samples.googlecode.com/svn/trunk/urlsigning/urlsigner.pl
-sub _decode_urlsafe_base64 {
+sub decode_urlsafe_base64 {
   my ($self, $content) = @_;
 
   $content =~ tr/-/\+/;
@@ -110,7 +108,7 @@ sub _decode_urlsafe_base64 {
   return MIME::Base64::decode_base64($content);
 }
 
-sub _encode_urlsafe{
+sub encode_urlsafe{
   my ($self, $content) = @_;
   $content =~ tr/\+/\-/;
   $content =~ tr/\//\_/;
@@ -118,20 +116,20 @@ sub _encode_urlsafe{
   return $content;
 }
 
-sub _make_signature {
+sub make_signature {
   my ($self, $uri) = @_;
 
   require Digest::HMAC_SHA1;
   require MIME::Base64;
 
-  my $key = $self->_decode_urlsafe_base64($self->{key});
+  my $key = $self->decode_urlsafe_base64($self->{key});
   my $to_sign = $uri->path_query;
 
   my $digest = Digest::HMAC_SHA1->new($key);
   $digest->add($to_sign);
   my $signature = $digest->b64digest;
 
-  return $self->_encode_urlsafe($signature);
+  return $self->encode_urlsafe($signature);
 }
 
 
@@ -155,7 +153,9 @@ Geo::Coder::Google::V3 provides a geocoding functionality using Google Maps API
 
 =head1 METHODS
 
-=head2 new
+=over 4
+
+=item new
 
   $geocoder = Geo::Coder::Google->new(apiver => 3);
   $geocoder = Geo::Coder::Google->new(apiver => 3, language => 'ru');
@@ -179,7 +179,7 @@ variables GMAP_CLIENT and GMAP_KEY before running 02_v3_live.t
 
   GMAP_CLIENT=your_id GMAP_KEY='your_key' make test
 
-=head2 geocode
+=item geocode
 
   $location = $geocoder->geocode(location => $location);
   @location = $geocoder->geocode(location => $location);
@@ -192,7 +192,7 @@ returns the 1st one in a scalar context.
 When you'd like to pass non-ascii string as a location, you should
 pass it as either UTF-8 bytes or Unicode flagged string.
 
-=head2 ua
+=item ua
 
 Accessor method to get and set UserAgent object used internally. You
 can call I<env_proxy> for example, to get the proxy information from
@@ -204,6 +204,8 @@ You can also set your own User-Agent object:
 
   $coder->ua( LWPx::ParanoidAgent->new );
 
+=back
+
 =head1 AUTHOR
 
 Tatsuhiko Miyagawa E<lt>miyagawa@bulknews.netE<gt>
